@@ -9,7 +9,7 @@ use POSIX qw(locale_h);
 setlocale(LC_NUMERIC,"C");
 
 
-$VERSION = '20.0';
+$VERSION = '21.0';
 
 # Some function might be excluded from export and assessment.
 our @EXCLUDED_FUNCTION = ('SQUIRREL_GET_ERROR_OFFSET');
@@ -203,7 +203,7 @@ sub _table_info
 	my $sql = "SELECT TABLE_NAME,TABLE_COMMENT,TABLE_TYPE,TABLE_ROWS,ROUND( ( data_length + index_length) / 1024 / 1024, 2 ) AS \"Total Size Mb\", AUTO_INCREMENT, ENGINE FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE='BASE TABLE' AND TABLE_SCHEMA = '$self->{schema}'";
 	$sql .= $self->limit_to_objects('TABLE', 'TABLE_NAME');
 	$sth = $self->{dbh}->prepare( $sql ) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	while (my $row = $sth->fetch) {
 		$row->[2] =~ s/^BASE //;
 		$comments{$row->[0]}{comment} = $row->[1];
@@ -247,11 +247,15 @@ sub _column_comments
 		$sql .= " WHERE TABLE_SCHEMA='$self->{schema}' ";
 	}
 	$sql .= "AND TABLE_NAME='$table' " if ($table);
-	$sql .= $self->limit_to_objects('TABLE','TABLE_NAME') if (!$table);
+	if (!$table) {
+		$sql .= $self->limit_to_objects('TABLE','TABLE_NAME');
+	} else {
+		@{$self->{query_bind_params}} = ();
+	}
 
 	my $sth = $self->{dbh}->prepare($sql) or $self->logit("WARNING only: " . $self->{dbh}->errstr . "\n", 0, 0);
 
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	my %data = ();
 	while (my $row = $sth->fetch) {
 		$data{$row->[2]}{$row->[0]} = $row->[1];
@@ -270,7 +274,11 @@ sub _column_info
 		$condition .= "AND TABLE_SCHEMA='$self->{schema}' ";
 	}
 	$condition .= "AND TABLE_NAME='$table' " if ($table);
-	$condition .= $self->limit_to_objects('TABLE', 'TABLE_NAME') if (!$table);
+	if (!$table) {
+		$condition .= $self->limit_to_objects('TABLE', 'TABLE_NAME');
+	} else {
+		@{$self->{query_bind_params}} = ();
+	}
 	$condition =~ s/^AND/WHERE/;
 
 	# TABLE_CATALOG            | varchar(512)        | NO   |     |         |       |
@@ -302,7 +310,7 @@ END
 	if (!$sth) {
 		$self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	}
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	# Expected columns information stored in hash 
 	# COLUMN_NAME,DATA_TYPE,DATA_LENGTH,NULLABLE,DATA_DEFAULT,DATA_PRECISION,DATA_SCALE,CHAR_LENGTH,TABLE_NAME,OWNER,VIRTUAL_COLUMN,POSITION,AUTO_INCREMENT,ENUM_INFO
@@ -327,7 +335,11 @@ sub _get_indexes
 
 	my $condition = '';
 	$condition = " FROM $self->{schema}" if ($self->{schema});
-	$condition .= $self->limit_to_objects('TABLE|INDEX', "`Table`|`Key_name`") if (!$table);
+	if (!$table) {
+		$condition .= $self->limit_to_objects('TABLE|INDEX', "`Table`|`Key_name`");
+	} else {
+		@{$self->{query_bind_params}} = ();
+	}
 	$condition =~ s/ AND / WHERE /;
 
 	my %tables_infos = ();
@@ -344,7 +356,7 @@ sub _get_indexes
 	# Retrieve all indexes for the given table
 	foreach my $t (keys %tables_infos) {
 		my $sth = $self->{dbh}->prepare("SHOW INDEX FROM $t $condition;") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-		$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+		$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 		my $i = 1;
 		while (my $row = $sth->fetch) {
@@ -396,7 +408,11 @@ sub _count_indexes
 
 	my $condition = '';
 	$condition = " FROM $self->{schema}" if ($self->{schema});
-	$condition .= $self->limit_to_objects('TABLE|INDEX', "`Table`|`Key_name`") if (!$table);
+	if (!$table) {
+		$condition .= $self->limit_to_objects('TABLE|INDEX', "`Table`|`Key_name`");
+	} else {
+		@{$self->{query_bind_params}} = ();
+	}
 	$condition =~ s/ AND / WHERE /;
 
 	my %tables_infos = ();
@@ -410,7 +426,7 @@ sub _count_indexes
 	# Retrieve all indexes for the given table
 	foreach my $t (keys %tables_infos) {
 		my $sth = $self->{dbh}->prepare("SHOW INDEX FROM $t $condition;") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-		$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+		$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 		my $i = 1;
 		while (my $row = $sth->fetch) {
@@ -506,7 +522,7 @@ sub _get_views
 	$str =~ s/ AND / WHERE /;
 
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my %ordered_view = ();
 	my %data = ();
@@ -563,7 +579,7 @@ sub _get_triggers
 
 	$str .= " ORDER BY EVENT_OBJECT_TABLE, TRIGGER_NAME";
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my @triggers = ();
 	while (my $row = $sth->fetch) {
@@ -594,7 +610,11 @@ sub _unique_key
 
 	my $condition = '';
 	$condition = " FROM $self->{schema}" if ($self->{schema});
-	$condition .= $self->limit_to_objects('TABLE|INDEX', "`Table`|`Key_name`") if (!$table);
+	if (!$table) {
+		$condition .= $self->limit_to_objects('TABLE|INDEX', "`Table`|`Key_name`");
+	} else {
+		@{$self->{query_bind_params}} = ();
+	}
 	$condition =~ s/ AND / WHERE /;
 
 	my %tables_infos = ();
@@ -606,7 +626,7 @@ sub _unique_key
 	# Retrieve all indexes for the given table
 	foreach my $t (keys %tables_infos) {
 		my $sth = $self->{dbh}->prepare("SHOW INDEX FROM $t $condition;") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-		$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+		$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 		my $i = 1;
 		while (my $row = $sth->fetch) {
@@ -657,6 +677,7 @@ sub _get_functions
 	# ROUTINE_NAME             | varchar(64)   | NO   |     |                     |       |
 	# ROUTINE_TYPE             | varchar(9)    | NO   |     |                     |       |
 	# DATA_TYPE                | varchar(64)   | NO   |     |                     |       |
+	#  or DTD_IDENTIFIER < 5.5 | varchar(64)   | NO   |     |                     |       |
 	# CHARACTER_MAXIMUM_LENGTH | int(21)       | YES  |     | NULL                |       |
 	# CHARACTER_OCTET_LENGTH   | int(21)       | YES  |     | NULL                |       |
 	# NUMERIC_PRECISION        | int(21)       | YES  |     | NULL                |       |
@@ -689,8 +710,12 @@ sub _get_functions
 	$str .= " " . $self->limit_to_objects('FUNCTION','ROUTINE_NAME');
 	$str =~ s/ AND / WHERE /;
 	$str .= " ORDER BY ROUTINE_NAME";
+	# Version below 5.5 do not have DATA_TYPE column it is named DTD_IDENTIFIER
+	if ($self->{db_version} lt '5.5.0') {
+		$str =~ s/,DATA_TYPE,/,DTD_IDENTIFIER,/;
+	}
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my %functions = ();
 	while (my $row = $sth->fetch) {
@@ -935,7 +960,7 @@ sub _list_all_funtions
 	$str =~ s/ AND / WHERE /;
 	$str .= " ORDER BY ROUTINE_NAME";
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my @functions = ();
 	while (my $row = $sth->fetch) {
@@ -1146,7 +1171,7 @@ sub _get_job
 	$str .= $self->limit_to_objects('JOB', 'EVENT_NAME');
 	$str .= " ORDER BY EVENT_NAME";
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my %data = ();
 	while (my $row = $sth->fetch) {
@@ -1171,7 +1196,7 @@ sub _get_dblink
 	$str =~ s/mysql.servers AND /mysql.servers WHERE /;
 
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my %data = ();
 	while (my $row = $sth->fetch) {
@@ -1209,7 +1234,7 @@ WHERE PARTITION_NAME IS NOT NULL AND SUBPARTITION_NAME IS NULL AND (PARTITION_ME
 	$str .= "ORDER BY TABLE_NAME,PARTITION_ORDINAL_POSITION\n";
 
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	my %parts = ();
 	my %default = ();
 	while (my $row = $sth->fetch) {
@@ -1249,7 +1274,7 @@ WHERE SUBPARTITION_NAME IS NOT NULL AND SUBPARTITION_EXPRESSION IS NOT NULL AND
 	}
 	$str .= " ORDER BY TABLE_NAME,PARTITION_ORDINAL_POSITION\n";
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	my %subparts = ();
 	my %default = ();
 	while (my $row = $sth->fetch) {
@@ -1290,7 +1315,7 @@ FROM INFORMATION_SCHEMA.PARTITIONS WHERE SUBPARTITION_NAME IS NULL AND PARTITION
 	$str .= " ORDER BY TABLE_NAME,PARTITION_NAME\n";
 
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my %parts = ();
 	while (my $row = $sth->fetch) {
@@ -1323,7 +1348,7 @@ FROM INFORMATION_SCHEMA.PARTITIONS WHERE SUBPARTITION_NAME IS NULL AND PARTITION
 	$str .= " ORDER BY TABLE_NAME,PARTITION_NAME\n";
 
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my %parts = ();
 	while (my $row = $sth->fetch) {
@@ -1408,7 +1433,7 @@ WHERE SUBPARTITION_NAME IS NULL AND (PARTITION_METHOD = 'RANGE' OR PARTITION_MET
 		$sql .= "\tAND TABLE_SCHEMA ='$self->{schema}'\n";
 	}
 	$sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	while ( my @row = $sth->fetchrow()) {
 		push(@{$infos{'TABLE PARTITION'}}, { ( name => $row[0], invalid => 0) });
 	}
@@ -1425,7 +1450,7 @@ WHERE SUBPARTITION_NAME IS NOT NULL
 		$sql .= "\tAND TABLE_SCHEMA ='$self->{schema}'\n";
 	}
 	$sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	while ( my @row = $sth->fetchrow()) {
 		push(@{$infos{'TABLE PARTITION'}}, { ( name => $row[0], invalid => 0) });
 	}
@@ -1450,7 +1475,7 @@ sub _get_privilege
 	$str .= " ORDER BY TABLE_NAME, GRANTEE";
 	my $error = "\n\nFATAL: You must be connected as an oracle dba user to retrieved grants\n\n";
 	my $sth = $self->{dbh}->prepare($str) or $self->logit($error . "FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	while (my $row = $sth->fetch) {
 		# Remove the host part of the user
 		$row->[0] =~ s/\@.*//;
@@ -1473,7 +1498,7 @@ sub _get_privilege
 	$str .= " " . $self->limit_to_objects('GRANT|TABLE|VIEW|FUNCTION|PROCEDURE|SEQUENCE', 'GRANTEE|TABLE_NAME|TABLE_NAME|TABLE_NAME|TABLE_NAME|TABLE_NAME');
 
 	$sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	while (my $row = $sth->fetch) {
 		$row->[0] =~ s/\@.*//;
 		$row->[0] =~ s/'//g;
@@ -1540,7 +1565,7 @@ WHERE TABLE_SCHEMA='$self->{schema}'
 	$sql .= " LIMIT $self->{top_max}" if ($self->{top_max});
 
         my $sth = $self->{dbh}->prepare( $sql ) or return undef;
-        $sth->execute or return undef;
+        $sth->execute(@{$self->{query_bind_params}}) or return undef;
 	while ( my @row = $sth->fetchrow()) {
 		$table_size{$row[0]} = $row[1];
 	}
@@ -1654,7 +1679,7 @@ sub _count_sequences
 	my $sql = "SELECT TABLE_NAME, AUTO_INCREMENT FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE='BASE TABLE' AND TABLE_SCHEMA = '$self->{schema}'";
 	$sql .= $self->limit_to_objects('TABLE', 'TABLE_NAME');
 	my $sth = $self->{dbh}->prepare( $sql ) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	while (my $row = $sth->fetch) {
 		push(@seqs, $row->[0]) if ($row->[1]);
 	}
@@ -1674,7 +1699,11 @@ sub _column_attributes
 		$condition .= "AND TABLE_SCHEMA='$self->{schema}' ";
 	}
 	$condition .= "AND TABLE_NAME='$table' " if ($table);
-	$condition .= $self->limit_to_objects('TABLE', 'TABLE_NAME') if (!$table);
+	if (!$table) {
+		$condition .= $self->limit_to_objects('TABLE', 'TABLE_NAME');
+	} else {
+		@{$self->{query_bind_params}} = ();
+	}
 	$condition =~ s/^AND/WHERE/;
 
 	# TABLE_CATALOG            | varchar(512)        | NO   |     |         |       |
@@ -1706,7 +1735,7 @@ END
 	if (!$sth) {
 		$self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	}
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my %data = ();
 	while (my $row = $sth->fetch) {
@@ -1731,7 +1760,7 @@ sub _list_triggers
 
 	$str .= " ORDER BY EVENT_OBJECT_TABLE, TRIGGER_NAME";
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my %triggers = ();
 	while (my $row = $sth->fetch) {
@@ -1893,7 +1922,7 @@ sub _get_security_definer
 	$str .= " ORDER BY ROUTINE_NAME";
 
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	while (my $row = $sth->fetch) {
 		next if (!$row->[0]);
